/*
   This program evaluates (interprets) an Abstract Syntax Tree (AST)
   that is defined by the following grammar.

       Prog ::= '(' 'prog' Define* Exp ')'

     Define ::= '(' 'define' VARIABLE NExp ')'
              | '(' 'define' FUNC-NAME Lambda ')'

     Lambda ::= '(' 'lambda' VARIABLE* Exp ')'

        Exp ::= Begin | Var | Print | While | NExp

      Begin ::= '(' 'begin' Exp  Exp+ ')'

        Var ::= '(' 'var' VARIABLE NExp ')'

      Print ::= '(' 'print' NExp ')'

      While ::= '(' 'while' NExp Exp ')'

       NExp ::= Apply | Set | If | AExp | BExp | Rand
              | Array | Index | SizeOf
              | VARIABLE | INTEGER | BOOLEAN | DOUBLE

      Apply ::= '(' 'apply' FUNC-NAME NExp* ')'
              | '(' 'apply' Lambda NExp* ')'    // applying an anonymous function

        Set ::= '(' 'set' VARIABLE NExp ')'
              | '(' 'set' Index NExp ')'        // set an array element

         If ::= '(' 'if' NExp Exp Exp ')'

       AExp ::= '(' BinOp NExp NExp ')'
              | '(' '+' NExp+ ')'
              | '(' '-' NExp ')'
              | '(' '-' NExp NExp ')'
              | '(' '*' NExp NExp+ ')'

     BinOp  ::= '/' | ' %' | '^'

       BExp ::= '(' BoolOp NExp NExp+ ')'
              | '(' '!' NExp ')'
              | '(' RelOp NExp NExp ')'
              | '('  EqOp NExp NExp ')'

     BoolOp ::= '&&' | '||'
     RelOp  ::= '<' | '>' | '<=' | '>='
      EqOp  ::= '==' | '!='

      Rand ::= '(' 'rand' DOUBLE ')'    // return a random number between 0 and DOUBLE

      Array ::= '(' 'array' 'dim' NExp ')'  // declares an array of size NExp
              | '(' 'array' NExp+ ')'       // an "array of doubles" literal

      Index ::= '(' 'index' NExp NExp ')'  // 1st operand is array, 2nd operand is integer

      SizeOf ::= '(' sizeOf NExp ')'       // operand must be an array

     DOUBLE ::= [0-9]+.[0-9]+   // that is, a number with a decimal point in it
    INTEGER ::= [0-9]+
    BOOLEAN ::= 'true' | 'false'
   VARIABLE ::= [a-zA-Z][a-zA-Z0-9]*
  FUNC-NAME ::= [a-zA-Z][a-zA-Z0-9]*

*/

public class Evaluate
{
   // Global environment for function definitions.
   private static FunctionEnvironment funcEnv;
   // Global environment for variable definitions.
   private static Environment globalEnv;


   public static Value evaluate(Tree tree) throws EvalException
   {
      funcEnv = new FunctionEnvironment();
      globalEnv = new Environment();

      return  evaluateProg( tree );
   }//evaluate()





}//Evaluate